from .generator import generate_coverage_matrix
from .preprocessor import preprocess
from .collapser import collapse

__version__ = "1.0.0"

__all__ = ("generate_coverage_matrix", "preprocess", "collapse")
